@@ -334,10 +334,9 @@ sub bgsend {
 
   my $pkt = $self->new_dns_packet($host, $type, $class);
 
-  my $data = $pkt->data;
   $self->connect_sock_if_reqd();
-  if (!$self->{sock}->send ($pkt->data, 0)) {
-    warn "dns: sendto() failed: $@";
+  if (!defined($self->{sock}->send($pkt->data, 0))) {
+    warn "dns: sendto() failed: $!";
     return;
   }
   my $id = $self->_packet_id($pkt);
@@ -402,6 +401,19 @@ sub poll_responses {
 
 ###########################################################################
 
+=item $res->bgabort()
+
+Call this to release pending requests from memory when aborting backgrounded requests
+
+=cut
+
+sub bgabort {
+  my ($self) = @_;
+  $self->{id_to_callback} = {};
+}
+
+###########################################################################
+
 =item $packet = $res->send($name, $type, $class)
 
 Emulates C<Net::DNS::Resolver::send()>.
