@@ -1,3 +1,19 @@
+# <@LICENSE>
+# Copyright 2004 Apache Software Foundation
+# 
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+# 
+#     http://www.apache.org/licenses/LICENSE-2.0
+# 
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+# </@LICENSE>
+
 =head1 NAME
 
 Mail::SpamAssassin::PerMsgLearner - per-message status (spam or not-spam)
@@ -6,12 +22,13 @@ Mail::SpamAssassin::PerMsgLearner - per-message status (spam or not-spam)
 
   my $spamtest = new Mail::SpamAssassin ({
     'rules_filename'      => '/etc/spamassassin.rules',
-    'userprefs_filename'  => $ENV{HOME}.'/.spamassassin.cf'
+    'userprefs_filename'  => $ENV{HOME}.'/.spamassassin/user_prefs'
   });
-  my $mail = Mail::SpamAssassin::NoMailAudit->new();
+  my $mail = $spamtest->parse();
 
-  my $status = $spamtest->learn ($mail);
-  ...
+  my $status = $spamtest->learn($mail,$id,$isspam,$forget);
+  my $didlearn = $status->did_learn();
+  $status->finish();
 
 
 =head1 DESCRIPTION
@@ -29,12 +46,13 @@ the learning process.
 package Mail::SpamAssassin::PerMsgLearner;
 
 use strict;
+use warnings;
 use bytes;
 
 use Mail::SpamAssassin;
-use Mail::SpamAssassin::AutoWhitelist;
 use Mail::SpamAssassin::PerMsgStatus;
 use Mail::SpamAssassin::Bayes;
+use Mail::SpamAssassin::Logger;
 
 use vars qw{
   @ISA
@@ -47,7 +65,7 @@ use vars qw{
 sub new {
   my $class = shift;
   $class = ref($class) || $class;
-  my ($main, $msg, $id) = @_;
+  my ($main, $msg) = @_;
 
   my $self = {
     'main'              => $main,
@@ -59,27 +77,25 @@ sub new {
 
   $self->{bayes_scanner} = $self->{main}->{bayes_scanner};
 
-  $id ||= $self->{msg}->get_header ("Message-Id");
-  $id ||= $self->{msg}->get_header ("Message-ID");
-  $id ||= 'no_id.$$.'.rand();
-  $id =~ s/[-\0\s\;\:]/_/gs;
-
-  $self->{id} = $id;
-
   bless ($self, $class);
   $self;
 }
 
 ###########################################################################
 
-=item $status->learn_spam()
-
-Learn the message as spam.
-
-=cut
+# $status->learn_spam($id)
+# 
+# Learn the message as spam.
+# 
+# C<$id> is an optional message-identification string, used internally
+# to tag the message.  If it is C<undef>, the Message-Id of the message
+# will be used.  It should be unique to that message.
+# 
+# This is a semi-private API; callers should use
+# C<$spamtest-E<gt>learn($mail,$id,$isspam,$forget)> instead.
 
 sub learn_spam {
-  my ($self) = @_;
+  my ($self, $id) = @_;
 
   if ($self->{main}->{learn_with_whitelist}) {
     $self->{main}->add_all_addresses_to_blacklist ($self->{msg});
@@ -88,43 +104,53 @@ sub learn_spam {
   # use the real message-id here instead of mass-check's idea of an "id",
   # as we may deliver the msg into another mbox format but later need
   # to forget it's training.
-  $self->{learned} = $self->{bayes_scanner}->learn (1, $self->{msg});
+  $self->{learned} = $self->{bayes_scanner}->learn (1, $self->{msg}, $id);
 }
 
 ###########################################################################
 
-=item $status->learn_ham()
-
-Learn the message as ham.
-
-=cut
+# $status->learn_ham($id)
+# 
+# Learn the message as ham.
+# 
+# C<$id> is an optional message-identification string, used internally
+# to tag the message.  If it is C<undef>, the Message-Id of the message
+# will be used.  It should be unique to that message.
+# 
+# This is a semi-private API; callers should use
+# C<$spamtest-E<gt>learn($mail,$id,$isspam,$forget)> instead.
 
 sub learn_ham {
-  my ($self) = @_;
+  my ($self, $id) = @_;
 
   if ($self->{main}->{learn_with_whitelist}) {
     $self->{main}->add_all_addresses_to_whitelist ($self->{msg});
   }
 
-  $self->{learned} = $self->{bayes_scanner}->learn (0, $self->{msg});
+  $self->{learned} = $self->{bayes_scanner}->learn (0, $self->{msg}, $id);
 }
 
 ###########################################################################
 
-=item $status->forget()
-
-Forget about a previously-learned message.
-
-=cut
+# $status->forget($id)
+# 
+# Forget about a previously-learned message.
+# 
+# C<$id> is an optional message-identification string, used internally
+# to tag the message.  If it is C<undef>, the Message-Id of the message
+# will be used.  It should be unique to that message.
+# 
+# This is a semi-private API; callers should use
+# C<$spamtest-E<gt>learn($mail,$id,$isspam,$forget)> instead.
 
 sub forget {
-  my ($self) = @_;
+  my ($self, $id) = @_;
 
   if ($self->{main}->{learn_with_whitelist}) {
     $self->{main}->remove_all_addresses_from_whitelist ($self->{msg});
   }
 
-  $self->{learned} = $self->{bayes_scanner}->forget ($self->{msg});
+  $self->{learned} = $self->{bayes_scanner}->forget ($self->{msg}, $id);
 }
 
 ###########################################################################
@@ -153,15 +179,11 @@ sub finish {
   delete $self->{main};
   delete $self->{msg};
   delete $self->{conf};
+  delete $self->{bayes_scanner};
 }
 
 ###########################################################################
 
-sub dbg { Mail::SpamAssassin::dbg (@_); }
-sub timelog { Mail::SpamAssassin::timelog (@_); }
-
-###########################################################################
-
 1;
 __END__
 
